#ifndef AVIFILE_READHANDLERS_H
#define AVIFILE_READHANDLERS_H

/**
 *
 * Interfaces that provide uniform access to data in AVI, local and
 * networked ASF files.
 *
 */

#include <formats.h>
#include <default.h>
#include <avifmt.h>
#include <string>
#include <vector>
#include <except.h>

class IMediaReadStream {
  public:
    static const framepos_t ERR = ~0U;

    virtual ~IMediaReadStream() {}
    /* Return structure describing main stream header */
    virtual HRESULT Header(AVIStreamHeader *pheader) const	= 0;

    /* not implemented */
    // virtual HRESULT Info(AVISTREAMINFO *pasi, int lSize) const     =0;

    /* return true if frame no. lFrame ( -1 means current frame ) is keyframe */
    virtual bool IsKeyFrame(framepos_t lFrame = ERR) const	= 0;

    /* seek to position lStart in stream */
    virtual HRESULT Seek(framepos_t lStart)			= 0;

    /* seek to time lTime */
    virtual HRESULT SeekTime(double time)			= 0;

    /* Read up to lSamples into user-specified buffer, starting from
       current position, and increase counter.
       For video stream lpBuffer may be 0, in which case lSamples
       should be 1 and function returns size of next video frame
       in plBytes. */
    virtual HRESULT Read(unsigned int lSamples, void *lpBuffer,
			 unsigned int cbBuffer, unsigned int *plBytes,
			 unsigned int *plSamples)		= 0;
    virtual HRESULT SkipFrame()					= 0;
    virtual HRESULT SkipTo(double time)				= 0;

    /* current stream position */
    virtual framepos_t Pos() const				= 0;

    /* positions of previous, next or nearest key frames related to
       frame lFrame ( -1 current frame )                                */
    virtual framepos_t PrevKeyFrame(framepos_t lFrame = ERR) const = 0;
    virtual framepos_t NextKeyFrame(framepos_t lFrame = ERR) const = 0;
    virtual framepos_t NearestKeyFrame(framepos_t lFrame = ERR) const = 0;

    virtual HRESULT SeekToPrevKeyFrame()			= 0;

    /* length of the stream in samples */
    virtual unsigned int Length() const				= 0;

    /* length of the stream in seconds */
    virtual double LengthTime() const				= 0;

    /* timestamp of sample no. lSample ( ERR = current sample ) */
    virtual double GetSampleTime(framepos_t lSample = ERR) const = 0;

    /* retrieval of stream-specific format data */
    virtual HRESULT FormatSize(unsigned int *plSize) const	= 0;
    virtual HRESULT ReadFormat(void *pFormat, unsigned int *plSize) const = 0;

    /* 1 cache full, 0 cache empty */
    virtual double CacheSize() const				= 0;

    /* clear the internal stream cache */
    virtual void ClearCache()					= 0;

    /* usually 'vids' or 'auds' */
    virtual fourcc_t FourCC() const				= 0;

    /* returns 0 for video ( frames of variable size ), >0 for audio */
    virtual unsigned int SampleSize() const			= 0;
};

class IMediaReadHandler {
public:
    virtual ~IMediaReadHandler() {}
    virtual IMediaReadStream *GetStream(fourcc_t fccType, int lParam) = 0;	//vids/auds, 0-...
    virtual HRESULT Header(MainAVIHeader* pheader) const 	= 0;

    /* All calls for network-oriented readers should be interruptible  */
    virtual void Interrupt() 					= 0;

    /* in case of asynchronous opening */
    virtual bool IsOpened() 					= 0;
    virtual bool IsValid()					= 0;
    virtual bool IsRedirector() 				= 0;
    virtual bool GetURLs(std::vector<std::string> &urls) 	= 0;
};

/*  Read handler for AVI file  */
IMediaReadHandler *CreateAviMediaReadHandler(const char *pszFile) throw(FatalError);

/*  Read handler for ASF/WMV/ASX file  */
IMediaReadHandler *CreateASFReadHandler(const char *pszFile) throw(FatalError);

#endif // AVIFILE_READHANDLERS_H
