#ifndef AVIFILE_IAVIPLAYER_H
#define AVIFILE_IAVIPLAYER_H

/********************************************************

    AVI player, ver. 0.6.0
    Copyright 2000 Eugene Kuznetsov  (divx@euro.ru)

*********************************************************/

#include <config.h>
#include <except.h>
#include <image.h>
#include <infotypes.h>

#ifndef WITHOUT_X
#include <X11/Xlib.h>
#endif
typedef void (*DRAWFUNC)(const void*, void* p = 0);
typedef void (*DRAWFUNC2)(const CImage*, void* p = 0);
typedef void (*KILLHANDLER)(int, void* p = 0);
typedef int (*AUDIOFUNC)(void*, int, void* p = 0);

class PlayerWidget;
class IRtConfig;

/**
 *
 *   This class is meant to contain all code that is
 * not related to graphical interface of AVI/ASF player: you provide
 * an interface ( buttons, windows, etc. ), this class
 * provides you with video data and performs audio playback.
 *
 *   NEVER call playback-controlling functions ( even indirectly
 * through event processing calls, such as qApp->processEvents
 * in Qt ) from its callbacks.
 *
 *    Usage:
 *
 *   Call CreateAviPlayer(), specifying file name of AVI/ASF file, bit depth
 * of physical screen and optional file name of subtitle file.
 * Catch FatalError if player can't be constructed. Set two callbacks:
 * kill handler ( will be called when player stops ) and draw callback
 * ( called each frame during playback and frequently if paused. Takes
 * non-zero pointer to video data or NULL if frame sholdn't be drawn ).
 *  Passed URL may refer to remote ( MMS://... ) ASF file. Such files are
 * opened asynchronously. It means that constructor only performs sanity
 * checking on the file name, starts actual opening process ( in another
 * thread ) and returns. You should periodically call isOpened() until
 * it returns true, which means that opening process has finished. After that,
 * you can determine if it was successful by calling isValid(). For local
 * files isOpened() and isValid() will return 'true' immediately after
 * successful construction.
 *   URL can point to ASX ( redirector ) file. For such file isRedirector()
 * will return 'true'. If this is the case, read list of redirected URLs
 * with getURLs(). After that, IAviPlayer object may be destroyed because
 * you can't do anything else with it.
 *  Prepare to draw video and call start(). Player immediately starts
 * playing. Its state can be controlled by calls play()/pause()/reseek(),
 * player can be stopped by stop() and started again by start().
 * Once again: don't try to call these functions from draw callback or
 * your program will burn in hell forever.
 *  When playing ASF files over the network and AVI files over slow
 * connections, player may occasionally get into 'buffering' state.
 * It means that player stops to output data and waits until its internal
 * buffers are refilled. You can determine if it's in buffering state
 * by calling getState(). It'll put buffer fill percentage into variable
 * pointed by its argument if buffering takes place ( here 1 corresponds
 * to full buffers and 0 to empty buffers ).
 *  Player can be destroyed with 'delete'. Deleting automatically stops
 * playback and frees all resources.
 *
 */
class IAviPlayer
{
public:
    enum State
    {
	Opening,
	Buffering,
	Playing,
	Stopped,
	Paused,
	Invalid
    };
    virtual ~IAviPlayer();
    virtual int SetColorSpace(int csp, bool test_only) =0;

    virtual bool IsOpened() const =0;
    virtual bool IsValid() const =0;
    virtual bool IsRedirector() const =0;
    virtual bool GetURLs(std::vector<std::string>& urls) const =0;

    virtual void SetDrawCallback(DRAWFUNC func, void* argument=0) =0;
    virtual void SetDrawCallback2(DRAWFUNC2 func, void* argument=0) =0;
    virtual void SetKillHandler(KILLHANDLER handler, void* argument=0) =0;
    virtual void SetAudioFunc(AUDIOFUNC func, void* argument=0) =0;

    virtual void Start() =0;
    virtual double Reseek(double pos) =0;
    virtual int ReseekExact(double pos) =0;	//nonzero if unable
    virtual int PageUp() =0;			//to next keyframe
    virtual int PageDown() =0;			//to prev keyframe ( the one before frame drawn last time )
    virtual void Play() =0;
    virtual void Pause(bool state) =0;
    virtual void Stop() =0;
    virtual int SetPlayMode(int realtime = 0) = 0;

    virtual bool IsPaused() const =0;		//true -> paused
    virtual bool IsStopped() const =0;		//true -> !playing && !paused
    virtual bool IsPlaying() const =0;		//true -> playing || paused

    virtual State GetState(double* percent) =0;

    virtual int GetWidth() const =0;
    virtual int GetHeight() const =0;
    virtual double GetFps() const =0;
    virtual const char* FileName() =0;

    virtual double GetPos() const =0;

    virtual const char* GetCurrentSubtitles() =0;
    virtual bool HasSubtitles() const =0;
    virtual double GetAudioLength() const =0;
    virtual double GetVideoLength() const =0;
    virtual const char* GetAudioFormat() const =0;
    virtual const char* GetVideoFormat() const =0;
    virtual double GetDrop() const =0;
    virtual double GetQuality() const =0;

    //
    // Audio/video sync correction
    //

    // argument in seconds
    virtual void SetAsync(float async) =0;
    virtual float GetAsync() const =0;

    //
    // Async applied to subtitles before playback
    //

    // argument in seconds
    virtual void SetSubAsync(float async) =0;
    virtual float GetSubAsync() const =0;
    // 0.0 mute, 1.0 maximum volume
    virtual void SetVolume(float volume) =0;
    virtual float GetVolume() const =0;

    virtual const CodecInfo& GetCodecInfo() const =0;
    virtual void Restart() =0;

    virtual IRtConfig* GetRuntimeConfig() =0;

    virtual void SetVideo(int) =0;
    virtual bool GetVideo() const =0;

    virtual void SetAudio(int) =0;
    virtual bool GetAudio() const =0;
};
/*
 This object has interface very similar to IAviPlayer, but
 it does drawing by itself. You do not need to provide
 pointer to kill handler, but you can still do it. Attempts
 to install draw callback will be ignored.
 Create an object with this interface by calling CreateAviPlayer2().
 Its arguments:
 parent - pointer to interface which handles a few specific
 key & mouse events in movie window.
 Optional. Pass it if you want to do this handling by yourself.
 dpy - pointer to opened X display. Optional. If it's non-NULL,
 movie window will be created on this display. If it's NULL,
 movie window won't be created at all.
 filename & subname - same as when constructing IAviPlayer.
 */

class IAviPlayer2: public IAviPlayer
{
public:
    virtual ~IAviPlayer2() {}
    virtual int Refresh()					=0;
    virtual int Resize(int& new_w, int& new_h)			=0;
    virtual int ToggleFullscreen(bool maximize = false)		=0;
};

IAviPlayer* CreateAviPlayer(const char* filename, int bitdepth,
			    const char* subfile = 0) throw(FatalError);
#ifdef WITHOUT_X
typedef int Display;
#endif
IAviPlayer2* CreateAviPlayer2(PlayerWidget*, Display*, const char* filename,
			      const char* subname) throw(FatalError);

double GetAvifileVersion();

#endif //AVIFILE_IAVIPLAYER_H
