#ifndef AVIFILE_IMAGE_H
#define AVIFILE_IMAGE_H

#include <cstdlib> //abs
#include <cstring>
#include <formats.h>
//#include <default.h>

/**
 * Structure that hides some details of proper BITMAPINFOHEADER
 * initialization methods.
 */
struct BitmapInfo:public BITMAPINFOHEADER
{
    int m_iColors[3];

    BitmapInfo() {}
    BitmapInfo(int width, int height, int bpp);
    BitmapInfo(const BITMAPINFOHEADER & hdr);
    BitmapInfo(const BitmapInfo & bi);
    BitmapInfo(const BitmapInfo * bi);
    static int BitCount(int csp);
    void SetBitFields16();
    void SetBitFields15();
    void SetRGB();
    void SetBits(int bits);
    void SetSpace(int csp);
    int Bpp() const;
    bool operator == (const BitmapInfo & bi) const
    {
	return (biWidth == bi.biWidth) && (biHeight == bi.biHeight)
	    && (biCompression == bi.biCompression) && (Bpp() == bi.Bpp());
    }
    bool IsRGB() const
    {
	return biCompression == 0 || biCompression == 3;
    }
    void Print() const;
};

/**
 * This class describes an uncompressed image in one of several supported
 * formats. These formats currently include RGB ( 15, 16, 24 and 32 bits )
 * and packed YUV ( non-subsampled 24-bit YUV and 16-bit YUY2 formats ).
 * It is capable of converting between all supported formats. Some of
 * conversions are optimized, but it is generally not recommended to use
 * these conversions in time-critical code.
 */
class CImage
{
    /* Copying disabled */
    CImage& operator = (CImage & e)
    {
	return *this;
    }
    CImage(const CImage & e)
    {
    }
protected:
    BitmapInfo* m_pInfo;
    unsigned char* m_pcData;
    bool m_bDataOwner;

    int m_iDepth;
    int m_iWidth;
    int m_iHeight;
    int m_iBpl;
    int m_iBpp;
    int m_iBytes;
    int m_iPixels;
    mutable int m_iRefcount;
    float m_fQuality;

    void fillMembers()
    {
	m_iDepth = m_pInfo->biBitCount;
	m_iBpp = (m_iDepth + 7) / 8;
	if (m_iDepth == 15)
	    m_iBpl = 2 * m_pInfo->biWidth;
	else
	    m_iBpl = (m_iDepth * m_pInfo->biWidth) / 8;
	m_iBytes = abs(m_iBpl * m_pInfo->biHeight);
	m_iPixels = abs(m_pInfo->biWidth * m_pInfo->biHeight);
	m_iWidth = abs(m_pInfo->biWidth);
	m_iHeight = abs(m_pInfo->biHeight);
    }
public:
    //
    //       non-conversion constructors
    //
    /**
     *  Creates new image in format 'header' from specified memory area.
     *  Either allocates its own memory area & copies src data into it, or reuses
     *  parent data.
     */
    CImage(const BitmapInfo* header, unsigned char *data = 0, bool copy = true);

    /* Creates 24-bit RGB image from 24-bit RGB 'data' */
    CImage(const unsigned char* data, int width, int height);

    /* Creates a copy of image 'im' */
    CImage(const CImage* im);

    // Conversion constructors

    /* Creates RGB image in depth 'depth' from image 'im' */
    CImage(const CImage* im, int depth);

    /*  Creates new image in format 'header' from image 'im' */
    CImage(const CImage* im, const BitmapInfo* header);

    void AddRef() const { m_iRefcount++; }
    void Release() const
    {
	m_iRefcount--;
	if (!m_iRefcount)
	    delete this;
    }

    virtual ~CImage();
    unsigned char *Data() { return m_pcData; }
    const unsigned char *Data() const { return m_pcData; }
    unsigned char *At(int i) { return m_pcData + i * m_iBpl; }
    const unsigned char *At(int i) const { return m_pcData + i * m_iBpl; }
    unsigned char *Offset(int i) { return m_pcData + i * m_iBpp; }
    const unsigned char *Offset(int i) const { return m_pcData + i * m_iBpp; }
    unsigned char *At(int i, int j) { return m_pcData + i * m_iBpp + j * m_iBpl; }
    int Width() const { return m_iWidth; }
    int Height() const { return m_iHeight; }
    int Bpp() const { return m_iBpp; }
    int Bpl() const { return m_iBpl; }
    int Depth() const { return m_iDepth; }
    int Bytes() const { return m_iBytes; }
    int Pixels() const { return m_iPixels; }
    bool Direction() const { return !(m_pInfo->biHeight < 0); }
    /**
     *  Is it in format 'hdr'?
     */
    bool IsFmt(const BitmapInfo * hdr) const { return (*m_pInfo == *hdr); }
    BitmapInfo *GetFmt();
    const BitmapInfo *GetFmt() const;

    /**
     * Translations 24-bit RGB <-> Non-subsampled packed 24-bit YUV
     * ( for image processing purposes )
     */
    void ToYUV();
    void ToRGB();

    /**
     * Fast MMX blur filter ( blur over the square with 2^range*2^range dimensions )
     * Blur(x); Blur(y, x) has the same effect as Blur(y)
     */
    void Blur(int range, int from = 0);

    /**
     * Writes 24-bit uncompressed bitmap with name 'filename'.
     */
    void Dump(const char *filename);

    /**
     * RGB<->BGR translation for 24-bit images
     */
    void ByteSwap();
    void *Getaddr();

    float GetQuality() const { return m_fQuality; }
    void SetQuality(float q) { m_fQuality = q; }

    static bool Supported(const BITMAPINFOHEADER & bi);
    static bool Supported(int csp, int bitcount);
    static int UnknownColorSpace(int csp);

  protected:

    /**
     * General way of converting is as follows.
     * a) There's one function for converting anything to 24-bit RGB and back.
     * b) There is a number of 'shortcuts' between formats, e.g. YUY2<->16-bit
     *  RGB.
     * c) If there's no shortcut between particular two formats, the conversion
     * is done in two steps through temporary image.
     */
    void convert(const unsigned char *from_data, const BitmapInfo * from_fmt);

    static void anyFromRgb24(unsigned char *to, const unsigned char *from, const BitmapInfo * from_fmt, bool flip_dir);
    static void anyToRgb24(unsigned char *to, const unsigned char *from, const BitmapInfo * to_fmt, bool flip_dir);

    static void rgb24ToRgb15(unsigned char *to, const unsigned char *from, int width, int height, bool flip_dir);
    static void rgb24ToRgb16(unsigned char *to, const unsigned char *from, int width, int height, bool flip_dir);
    static void rgb24ToRgb32(unsigned char *to, const unsigned char *from, int width, int height, bool flip_dir);
    static void rgb24ToYuv(unsigned char *to, const unsigned char *from, int width, int height, bool flip_dir);
    static void rgb24ToYuy2(unsigned char *to, const unsigned char *from, int width, int height, bool flip_dir);
    static void rgb24ToYv12(unsigned char *to, const unsigned char *from, int width, int height, bool flip_dir);

    static void rgb15ToRgb24(unsigned char *to, const unsigned char *from, int width, int height, bool flip_dir);
    static void rgb16ToRgb24(unsigned char *to, const unsigned char *from, int width, int height, bool flip_dir);
    static void rgb32ToRgb24(unsigned char *to, const unsigned char *from, int width, int height, bool flip_dir);
    static void yuvToRgb24(unsigned char *to, const unsigned char *from, int width, int height, bool flip_dir);
    static void yuy2ToRgb24(unsigned char *to, const unsigned char *from, int width, int height, bool flip_dir);
    static void yv12ToRgb24(unsigned char *to, const unsigned char *from, int width, int height, bool flip_dir);

    /* Fills memory area of 16-bit RGB image from pointer to 15-bit RGB data
     * Has MMX & non-MMX versions
     */
    static void rgb15ToRgb16(unsigned char *to, const unsigned char *from, int width, int height, bool flip_dir);
    static void yv12ToYuy2(unsigned char *to, const unsigned char *from, int width, int height, bool flip_dir);
    static void yuy2ToYv12(unsigned char *to, const unsigned char *from, int width, int height, bool flip_dir);
};

struct yuv;

struct lookuptable
{
    int m_plY[256];
    int m_plRV[256];
    int m_plGV[256];
    int m_plGU[256];
    int m_plBU[256];
    lookuptable()
    {
        for(int i=0; i<256; i++)
        {
    	    m_plY[i]=298*(i-16);
	    m_plRV[i]=408*(i-128);
    	    m_plGV[i]=-208*(i-128);
	    m_plGU[i]=-100*(i-128);
    	    m_plBU[i]=517*(i-128);
        }
    }
};
/**
 * Structure that represents 24-bit RGB pixel.
 */
struct col
{
    static lookuptable t;
    unsigned char b,g,r;
    col(){}
    col(unsigned char _b, unsigned char _g, unsigned char _r)
	:b(_b),g(_g),r(_r)
    {}
    inline col(yuv YUV);
    int CREF() const;
    unsigned int Y() const
    {
	int _Y=(16853l * r) + (33055l * g) + (6392l * b) + 0x8000;
	return (_Y>>16)+16;
    }
};

/**
 * Structure that represents 24-bit ( 888 ) YUV pixel.
 */
struct yuv
{
    unsigned char Y, 
		 Cb,/* aka U */
		 Cr;/* aka V */
    yuv() {}
    yuv(unsigned char _Y, unsigned char _Cb, unsigned char _Cr)
	:Y(_Y), Cb(_Cb), Cr(_Cr) {}
    yuv(col Col)
    {
	int _Y,_Cb,_Cr;
	_Y=66*Col.r+129*Col.g+25*Col.b+0x1000;
	_Cb=-38*Col.r-74*Col.g+112*Col.b+0x8000;
	_Cr=112*Col.r-94*Col.g-18*Col.b+0x8000;
	if(_Y<0x1000)_Y=0x1000;
	else if(_Y>0xef00)_Y=0xef00;
	if(_Cb<0x1000)_Cb=0x1000;
	else if(_Cb>0xef00)_Cb=0xef00;
	if(_Cr<0x1000)_Cr=0x1000;
        else if(_Cr>0xef00)_Cr=0xef00;

	Y=_Y>>8;
	Cb=_Cb>>8;
	Cr=_Cr>>8;
    }
};

inline col::col(yuv YUV)
{
    int R=t.m_plY[YUV.Y]+t.m_plRV[YUV.Cr];
    int G=t.m_plY[YUV.Y]+t.m_plGV[YUV.Cr]+t.m_plGU[YUV.Cb];
    int B=t.m_plY[YUV.Y]+t.m_plBU[YUV.Cb];
    if(B<0)B=0;
    else if(B>0xff00)B=0xff00;
    if(G<0)G=0;
    else if(G>0xff00)G=0xff00;
    if(R<0)R=0;
    else if(R>0xff00)R=0xff00;
    r=R>>8;
    g=G>>8;
    b=B>>8;
}

#endif // AVIFILE_IMAGE_H
