#ifndef AVIFILE_IVIDEODECODER_H
#define AVIFILE_IVIDEODECODER_H

/********************************************************

	Video decoder interface
	Copyright 2000 Eugene Kuznetsov  (divx@euro.ru)

*********************************************************/

#include <formats.h>
#include <default.h>
#include <infotypes.h>
#include <image.h>

// move bitmapinfo to separate file class CImage;
#ifndef WIN32
#include <pthread.h>
#endif
/**
 *
 *  Class for video decompression.
 *
 *  Usage:
 *    
 *  IVideoDecoder::Create() call tries to load video codec
 *  corresponding to your format, and throws FatalError if something goes wrong. 
 *  Start() starts decompression and Stop() stops it.
 *
 * SetDestFmt() sets desired bit depth and color space of output picture. 
 * Returns zero on success, -1 if format is unsupported and 
 * +1 if there was no 'native' support for the depth and library
 * is going to perform internal conversion of format. Most decoders 
 * support depths 15, 16, 24 or 32. Many of them also allow to use
 * YUV formats. You can check if your particular decoder is able to decode
 * into the particular YUV format by calling GetCapabilities(),
 * which returns bitwise OR of supported formats.
 *
 * DecodeFrame() decodes a single frame. Returns zero on success, any
 * other value on failure. Nonzero here is not fatal ( maybe it was
 * just one defective bit in file, after next keyframe everything 
 * will be OK ).
 *
 * SetDirection() allows to 'flip' output picture. By default picture 
 * is flipped ( DestFmt().biHeight<0 ), so that output image can
 * be simply memcpy()'ed to the X screen. It should be turned on
 * if image will be used for recompression, because some video encoders
 * will refuse to encode images with negative biHeight. Call it before
 * Start().
 *
 * Some decoders allow tweaking of their parameters. There are two ways
 * of doing it. 1) Use Get/SetExtendedAttr() to store parameter values
 * in registry. Decoders read these values during Start(), so you should
 * perform Restart() if you need to change parameters in run-time. 
 *
 * There's more convenient way, which is supported by a few DirectShow
 * codecs. If dynamic_cast<IRtConfig*>(decoder) gives you nonzero, you can
 * use IRtConfig methods to adjust picture properties between Start() and
 * Stop(). These adjustments are applied from the first frame following 
 * method call.
 *
 * List of valid parameters is available as GetCodecInfo().decoder_info.
 * See include/infotypes.h for details on accessing it.
 *
 */
class IRtConfig
{
public:
    virtual HRESULT GetValue(const char*, int&)		=0;
    virtual HRESULT SetValue(const char*, int)		=0;
};

class frame; // declared & defined bellow

class IVideoDecoder
{
public:
#ifndef WIN32
    static const int VBUFSIZE = 15;
    static const int QMARKHI = VBUFSIZE - 2;
    static const int QMARKLO = QMARKHI - 10;
    static const int DMARKHI = 6;
    static const int DMARKLO = 4;
#endif
    enum CAPS
    {
	CAP_NONE = 0,
	CAP_YUY2 = 1,
	CAP_YV12 = 2,
	CAP_IYUV = 4,
	CAP_UYVY = 8,
	CAP_YVYU = 16
    };
    IVideoDecoder(const CodecInfo& info, const BITMAPINFOHEADER& format);
    virtual ~IVideoDecoder();

    /* Functions which are implemented by each decoder */
    virtual void Start()						=0;
    virtual void Stop()							=0;
protected:
    /* Performs actual decoding */
    virtual int DecodeInternal(char* src, int size, 
	int is_keyframe, CImage* pImage)				=0;
public:	
    virtual int SetDestFmt(int bits = 24, int csp = 0) 			=0;

    /* Functions which may be reimplemented */
    virtual int SetDirection(int dir);
    virtual CAPS GetCapabilities() const { return CAP_NONE; }
    virtual void Restart(){Stop();Start();}
    /* sets internal variable 'realtime' to rt if decoder supports caching */
    virtual int SetRealtime(int rt) { return -1; }

    /* Functions which probably won't be overloaded */
    
    /**
     * Returns pointer to last decoded image. Future calls to DecodeFrame()
     * may overwrite the area contained in this object. Call CImage::release()  
     * when the image is not needed any more.
     * There's a possibility that underlying decoder is not capable of decoding
     * directly into your format, but CImage offers the necessary conversion.
     * In this case, the conversion will take place inside call to GetFrame().
     */
    virtual CImage* GetFrame();
    /**
     * Passes the data to decoder.
     * 'render' flag signals decoder if the frame will be rendered.
     * If it is 'false', decoder may speed up the decoding process 
     * by not filling CImage object. This feature is available only
     * for a few decoders, such as Indeo Video 5.0.
     */
    virtual int DecodeFrame(char* src, int size,
			    int is_keyframe, bool render = true );
    virtual int DropFrame() const;
    virtual framepos_t GetPos() const;
    virtual double GetTime() const;
    virtual int FlushCache();
    virtual int SetTimePos(double time, framepos_t pos);
    virtual const BITMAPINFOHEADER& DestFmt() {
	return (const BITMAPINFOHEADER&)m_obh;
    }
    virtual const CodecInfo& GetCodecInfo() const {
	return record;
    }
protected:
    const CodecInfo& record;
#ifndef WIN32
    frame *vbuf[VBUFSIZE];
    int decpos;
    int playpos;
    int qual;
    int filling;
    int realtime; // should we do precaching (or even change Quality on the fly)
    mutable pthread_mutex_t playposmut;
    mutable pthread_mutex_t decposmut;
#endif
    
    BITMAPINFOHEADER* m_bh;		//format of input data
    BitmapInfo m_decoder;		//format of decoder output
    BitmapInfo m_obh;			//format of returned frames
    CImage* m_outFrame;
};

//
// everything is inline here - its short - and its never used by
// user program so we don't have to worry about library incompatibily
// this class could be defined in some internal header file too

class frame
{
    CImage *m_pData;
    double m_dTime;
    framepos_t m_uiPos;
    bool m_bDrop;
public:
    frame() : m_pData(0), m_dTime(-1), m_uiPos(0), m_bDrop(0) {}
    ~frame()
    {
	if (m_pData)
	    m_pData->Release();
    }
    CImage* getData() { return m_pData; }
    bool getDrop() { return m_bDrop; }
    framepos_t getPos() { return m_uiPos; }
    double getTime() { return m_dTime; }

    void setData(CImage* newData)
    {
	if (m_pData)
	    m_pData->Release();
        m_pData = newData;
    }
    void setDataQuality(float q)
    {
	if (m_pData)
            m_pData->SetQuality(q);
    }
    float getDataQuality()
    {
	if(m_pData)
	    return m_pData->GetQuality();
	return 0.f;
    }
    void setDrop(bool d)
    {
	m_bDrop = d;
    }
    void setPos(framepos_t p)
    {
	m_uiPos = p;
    }
    void setTime(double t)
    {
	m_dTime = t;
    }
};

#endif // AVIFILE_IVIDEODECODER_H
/*
vim: tabstop=8
*/
